"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleHook = void 0;
const babel_plugin_macros_1 = require("babel-plugin-macros");
const utils_1 = require("./utils");
function handleHook(params) {
    var _a, _b;
    const { path, program, t } = params;
    const parent = path.parentPath;
    if (!(parent === null || parent === void 0 ? void 0 : parent.isCallExpression()))
        throw new babel_plugin_macros_1.MacroError(`useTailwindStyles can only be used as a function call.`);
    // Replace with imported version
    path.replaceWith(t.memberExpression(t.identifier(program.state.importIdentifier), t.identifier("useTailwindStyles")));
    const args = parent.get("arguments");
    const arrowFunc = args[0];
    if (!arrowFunc || !arrowFunc.isArrowFunctionExpression())
        throw new babel_plugin_macros_1.MacroError(`useTailwindStyles expects an inline arrow function as first argument.`);
    const tw = arrowFunc.get("params.0");
    if (!(tw === null || tw === void 0 ? void 0 : tw.isIdentifier()))
        throw new babel_plugin_macros_1.MacroError(`useTailwindStyles' arrow function expects a non-spread first argument.`);
    const references = (_b = (_a = arrowFunc.scope.getBinding(tw.node.name)) === null || _a === void 0 ? void 0 : _a.referencePaths) !== null && _b !== void 0 ? _b : [];
    if (!references.length)
        return;
    const useStylesIdentifier = program.scope.generateUid("useStyles");
    const createStylesPath = (0, utils_1.addCreateUseTailwindStyles)(Object.assign(Object.assign({}, params), { identifier: useStylesIdentifier }));
    // Find the surrounding function and convert body to block statement if necessary.
    const parentFunction = (0, utils_1.findParentFunction)(Object.assign(Object.assign({}, params), { path }));
    const body = parentFunction.get("body");
    // Won't happen
    if (!body.isBlockStatement())
        throw new babel_plugin_macros_1.MacroError("Expected surrounding function to contain block statement.");
    const stylesIdentifier = body.scope.generateUid("styles");
    (0, utils_1.addUseStylesCall)(Object.assign(Object.assign({}, params), { path: body, identifier: stylesIdentifier, useStylesIdentifier }));
    // Add styles to dependency array
    if (args.length < 2) {
        parent.pushContainer("arguments", t.arrayExpression([t.identifier(stylesIdentifier)]));
    }
    else {
        const deps = args[1];
        if (!deps.isArrayExpression())
            throw new babel_plugin_macros_1.MacroError(`useTailwindStyles expects an inline array expression of dependencies as second argument.`);
        deps.unshiftContainer("elements", t.identifier(stylesIdentifier));
    }
    /* eslint-disable-next-line no-shadow */
    references.forEach((path) => {
        const taggedTemplate = path.parentPath;
        if (!(taggedTemplate === null || taggedTemplate === void 0 ? void 0 : taggedTemplate.isTaggedTemplateExpression()))
            throw new babel_plugin_macros_1.MacroError("tw can only be used as template literals like tw`px-8`.");
        const quasis = taggedTemplate.get("quasi.quasis");
        if (quasis.length !== 1)
            throw new babel_plugin_macros_1.MacroError(`tw can only accept a static string, it does not support interpolated expressions.`);
        const style = quasis[0].node.value.raw;
        const styleIdentifier = (0, utils_1.addStyleRule)(Object.assign(Object.assign({}, params), { style, createStylesPath }));
        taggedTemplate.replaceWith(t.memberExpression(t.identifier(stylesIdentifier), t.stringLiteral(styleIdentifier), true));
    });
}
exports.handleHook = handleHook;
