"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const babel_plugin_macros_1 = require("babel-plugin-macros");
const fs_1 = __importDefault(require("fs"));
const path_1 = require("path");
const utils_1 = require("./utils");
const handle_tw_prop_1 = require("./handle-tw-prop");
const handle_hook_1 = require("./handle-hook");
const handle_lib_import_1 = require("./handle-lib-import");
const resolve_tailwind_config_1 = require("./resolve-tailwind-config");
const macro = (params) => {
    var _a, _b;
    const _c = params.references, { 
    // eslint-disable-next-line @typescript-eslint/no-unused-vars
    default: _default, useTailwindStyles: useTailwindStylesReferences = [] } = _c, libImports = __rest(_c, ["default", "useTailwindStyles"]), { state, babel: { types: t }, config = {} } = params;
    const program = state.file.path;
    (0, utils_1.addImport)(Object.assign(Object.assign({}, params), { t, path: program, program }));
    // Resolve custom tailwind config
    const sourceRoot = (_a = state.file.opts.sourceRoot) !== null && _a !== void 0 ? _a : ".";
    const configPath = (0, path_1.resolve)(sourceRoot, (_b = config.config) !== null && _b !== void 0 ? _b : "./tailwind.config.js");
    if (config.config && !fs_1.default.existsSync(configPath))
        throw new babel_plugin_macros_1.MacroError(`Couldn't find custom tailwind config at ${configPath}.`);
    program.state.tailwindConfig = (0, resolve_tailwind_config_1.resolveTailwindConfig)(configPath);
    /**
     * Traverse and collect all tw props on the surrounding component function in a first run.
     */
    program.traverse({
        JSXElement(path) {
            var _a;
            const allAttributes = path.get("openingElement").get("attributes");
            const jsxAttributes = allAttributes.filter((a) => a.isJSXAttribute());
            const twProp = jsxAttributes.find((a) => a.node.name.name === "tw");
            // Nothing to do
            if (!twProp)
                return;
            const component = (0, utils_1.findParentFunction)(Object.assign(Object.assign({}, params), { path,
                t,
                program }));
            if (!component.state.stylesIdentifier)
                component.state.stylesIdentifier =
                    component.scope.generateUid("tailwindStyles");
            if (!component.state.useStylesIdentifier)
                component.state.useStylesIdentifier =
                    component.scope.generateUid("useStyles");
            if (!((_a = component.state) === null || _a === void 0 ? void 0 : _a.createStylesPath)) {
                const createStylesPath = (0, utils_1.addCreateUseTailwindStyles)(Object.assign(Object.assign({}, params), { t,
                    program,
                    path, identifier: component.state.useStylesIdentifier }));
                component.state.createStylesPath = createStylesPath;
            }
            const body = component.get("body");
            if (body.isBlockStatement() && !component.state.useStylesPath) {
                const useStyles = (0, utils_1.addUseStylesCall)(Object.assign(Object.assign({}, params), { t,
                    program, path: body, identifier: component.state.stylesIdentifier, useStylesIdentifier: component.state.useStylesIdentifier }));
                component.state.useStylesPath = useStyles;
            }
            const stylesObject = component.state.useStylesPath.get("declarations.0.init.arguments.0");
            component.state.useStylesArgumentPath = stylesObject;
            (0, handle_tw_prop_1.handleTwProp)(Object.assign(Object.assign({}, params), { path: twProp, t, program, component }));
        },
    });
    useTailwindStylesReferences.forEach((path) => (0, handle_hook_1.handleHook)(Object.assign(Object.assign({}, params), { path, program, t })));
    Object.values(libImports)
        .flat()
        .forEach((path) => (0, handle_lib_import_1.handleLibImport)(Object.assign(Object.assign({}, params), { path, program, t })));
    program.scope.crawl();
};
exports.default = (0, babel_plugin_macros_1.createMacro)(macro, {
    configName: "reactNativeTailwind",
});
