"use strict";
var __rest = (this && this.__rest) || function (s, e) {
    var t = {};
    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
        t[p] = s[p];
    if (s != null && typeof Object.getOwnPropertySymbols === "function")
        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                t[p[i]] = s[p[i]];
        }
    return t;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.addUseStylesCall = exports.addCreateUseTailwindStyles = exports.addResponsiveId = exports.moveTwPropToStyle = exports.addStyleRule = exports.addImport = exports.astify = exports.findParentFunction = exports.generateUniqueStyleIdentifier = void 0;
const babel_plugin_macros_1 = require("babel-plugin-macros");
const parser_1 = __importDefault(require("@babel/parser"));
const template_1 = __importDefault(require("@babel/template"));
const non_secure_1 = require("nanoid/non-secure");
const transform_styles_1 = require("./transform-styles");
const SPREAD_ID = "__spread__";
const COMPUTED_ID = "__computed__";
/**
 * Generates a new pseudo-random style identifier.
 */
function generateUniqueStyleIdentifier() {
    return (0, non_secure_1.nanoid)(10);
}
exports.generateUniqueStyleIdentifier = generateUniqueStyleIdentifier;
/**
 * Returns the enclosing function (aka component) or throws if none is found.
 */
function findParentFunction(_a) {
    var { path, t } = _a, args = __rest(_a, ["path", "t"]);
    const functionParent = path.getFunctionParent();
    if (!functionParent)
        throw new babel_plugin_macros_1.MacroError(`You can only use "tw" and "useTailwindStyles" inside functions and function components.`);
    if (!functionParent.state)
        functionParent.state = { twRules: [], hasHook: true };
    convertImmediateJSXReturnToBlockStatement(Object.assign(Object.assign({}, args), { t, path: functionParent }));
    return functionParent;
}
exports.findParentFunction = findParentFunction;
/**
 * Converts an arrow function with immediate return into a block statement with explicit return.
 */
function convertImmediateJSXReturnToBlockStatement({ path, t, }) {
    const body = path.node.body;
    if (!t.isBlockStatement(body)) {
        path.set("body", t.blockStatement([t.returnStatement(body)]));
    }
}
/**
 * Convert plain js into babel ast
 * Credits: https://github.com/ben-rogerson/twin.macro/blob/master/src/macroHelpers.js#L75
 */
function astify(literal, t) {
    if (literal === null) {
        return t.nullLiteral();
    }
    switch (typeof literal) {
        case "function":
            return t.unaryExpression("void", t.numericLiteral(0), true);
        case "number":
            return t.numericLiteral(literal);
        case "boolean":
            return t.booleanLiteral(literal);
        case "undefined":
            return t.unaryExpression("void", t.numericLiteral(0), true);
        case "string":
            if (literal.startsWith(COMPUTED_ID)) {
                return parser_1.default.parseExpression(literal.slice(COMPUTED_ID.length));
            }
            return t.stringLiteral(literal);
        default:
            // Assuming literal is an object
            if (Array.isArray(literal)) {
                return t.arrayExpression(literal.map((x) => astify(x, t)));
            }
            return t.objectExpression(objectExpressionElements(literal, t, "spreadElement"));
    }
}
exports.astify = astify;
/**
 * Credits: https://github.com/ben-rogerson/twin.macro/blob/master/src/macroHelpers.js#L54
 */
function objectExpressionElements(literal, t, spreadType) {
    return Object.keys(literal)
        .filter((k) => {
        return typeof literal[k] !== "undefined";
    })
        .map((k) => {
        if (k.startsWith(SPREAD_ID)) {
            return t[spreadType](parser_1.default.parseExpression(literal[k]));
        }
        const computed = k.startsWith(COMPUTED_ID);
        const key = computed
            ? parser_1.default.parseExpression(k.slice(12))
            : t.stringLiteral(k);
        return t.objectProperty(key, astify(literal[k], t), computed);
    });
}
/**
 * Adds the lib import to the top of the file. Skips if it was already added.
 */
function addImport({ program, t, source }) {
    var _a;
    if ((_a = program.state) === null || _a === void 0 ? void 0 : _a.didImport)
        return;
    const identifier = program.scope.generateUid("ReactNativeTailwindMacro");
    program.unshiftContainer("body", t.importDeclaration([t.importNamespaceSpecifier(t.identifier(identifier))], t.stringLiteral(source + "/dist/exports")));
    if (!program.state)
        program.state = {};
    program.state.importIdentifier = identifier;
    program.state.didImport = true;
}
exports.addImport = addImport;
/**
 * Adds a style rule to the `createUseTailwindStyles` call.
 * @returns Generated style identifier
 */
function addStyleRule({ createStylesPath, style, t, program, }) {
    const id = generateUniqueStyleIdentifier();
    const object = createStylesPath.get("declarations.0.init.arguments.0");
    object.pushContainer("properties", t.objectProperty(t.stringLiteral(id), astify((0, transform_styles_1.transformStyles)(style, program.state.tailwindConfig), t)));
    return id;
}
exports.addStyleRule = addStyleRule;
/**
 * Moves the translated tw prop calls to the styles prop.
 */
function moveTwPropToStyle({ path, identifier, component, t, }) {
    const jsxElement = path.parentPath;
    const allAttributes = jsxElement.get("attributes");
    const jsxAttributes = allAttributes.filter((a) => a.isJSXAttribute());
    const twPropIndex = jsxAttributes.findIndex((a) => a.node.name.name === "tw");
    const stylePropIndex = jsxAttributes.findIndex((a) => a.node.name.name === "style");
    const mode = twPropIndex > stylePropIndex ? "append" : "prepend";
    const twStyleExpression = t.memberExpression(t.identifier(component.state.stylesIdentifier), t.stringLiteral(identifier), true);
    if (stylePropIndex < 0) {
        /**
         * There is no style prop, simply add a new one.
         */
        path.insertBefore(t.jsxAttribute(t.jsxIdentifier("style"), t.jsxExpressionContainer(twStyleExpression)));
    }
    else {
        /**
         * If styles is not an array, convert it into one.
         * Otherwise, simply append or prepend style identifier.
         */
        const styleProp = jsxAttributes[stylePropIndex];
        const styleExpression = styleProp.get("value.expression");
        if (styleExpression.isArrayExpression()) {
            if (mode === "append") {
                styleExpression.pushContainer("elements", twStyleExpression);
            }
            else {
                styleExpression.unshiftContainer("elements", twStyleExpression);
            }
        }
        else {
            const literal = styleExpression.node;
            styleProp.replaceWith(t.jsxAttribute(t.jsxIdentifier("style"), t.jsxExpressionContainer(t.arrayExpression(mode === "append"
                ? [literal, twStyleExpression]
                : [twStyleExpression, literal]))));
        }
    }
    // Remove tw prop
    path.remove();
}
exports.moveTwPropToStyle = moveTwPropToStyle;
/**
 * Adds the responsive ids to the dataSet tw property.
 */
function addResponsiveId({ path, identifier, component, program, t, }) {
    const jsxElement = path.get("openingElement");
    const dataSetProp = jsxElement
        .get("attributes")
        .find((a) => a.isJSXAttribute() && a.node.name.name === "dataSet");
    const objectProperty = t.objectProperty(t.identifier("tw"), t.memberExpression(t.memberExpression(t.identifier(component.state.stylesIdentifier), t.stringLiteral(identifier), true), t.identifier("id")));
    if (!dataSetProp) {
        jsxElement.pushContainer("attributes", t.jsxAttribute(t.jsxIdentifier("dataSet"), t.jsxExpressionContainer(t.objectExpression([objectProperty]))));
    }
    else {
        const expression = dataSetProp.get("value.expression");
        if (!expression.isObjectExpression())
            throw new babel_plugin_macros_1.MacroError("dataSet must be an inline object definition for react-native-tailwind.macro to apply responsive styles.");
        const twProperty = expression
            .get("properties")
            .find((prop) => {
            var _a;
            return prop.isObjectProperty() &&
                ((_a = prop.get("key").node) === null || _a === void 0 ? void 0 : _a.name) === "tw";
        });
        if (twProperty) {
            const value = twProperty.get("value");
            if (!value.isExpression() &&
                !value.isJSXNamespacedName() &&
                !value.isSpreadElement())
                throw new babel_plugin_macros_1.MacroError("Cannot augment dataSet tw property.");
            value.replaceWith(t.callExpression(t.memberExpression(t.identifier(program.state.importIdentifier), t.identifier("joinResponsiveIds")), [
                value.node,
                t.memberExpression(t.memberExpression(t.identifier(component.state.stylesIdentifier), t.stringLiteral(identifier), true), t.identifier("id")),
            ]));
        }
        else
            expression.pushContainer("properties", objectProperty);
    }
}
exports.addResponsiveId = addResponsiveId;
function addCreateUseTailwindStyles({ program, identifier, }) {
    const [createStylesPath] = program.pushContainer("body", (0, template_1.default)("const %%useStyles%% = %%import%%.createUseTailwindStyles({})")({
        useStyles: identifier,
        import: program.state.importIdentifier,
    }));
    return createStylesPath;
}
exports.addCreateUseTailwindStyles = addCreateUseTailwindStyles;
function addUseStylesCall({ path, identifier, useStylesIdentifier, }) {
    const [useStyles] = path.unshiftContainer("body", (0, template_1.default)("const %%styles%% = %%useStyles%%()")({
        styles: identifier,
        useStyles: useStylesIdentifier,
    }));
    return useStyles;
}
exports.addUseStylesCall = addUseStylesCall;
