"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInitialColorScheme = exports.setColorSchemeCookie = exports.getColorSchemeFromCookie = void 0;
const react_native_1 = require("react-native");
/**
 * Tries to retrieve the color scheme from a persisted cookie on web.
 */
const getColorSchemeFromCookie = () => {
    var _a, _b;
    if (typeof document !== "undefined") {
        const cookie = (_b = (_a = document.cookie) === null || _a === void 0 ? void 0 : _a.match("(^|;)\\s*__rntwm_color_scheme\\s*=\\s*([^;]+)")) === null || _b === void 0 ? void 0 : _b.pop();
        if (cookie === "light" || cookie === "dark")
            return cookie;
    }
    return undefined;
};
exports.getColorSchemeFromCookie = getColorSchemeFromCookie;
/**
 * Tries to set the color scheme as a cookie on web.
 */
const setColorSchemeCookie = (colorScheme) => {
    if (typeof document !== "undefined") {
        document.cookie = `__rntwm_color_scheme=${colorScheme};expires=${new Date(
        // Expire in 10 years, practically never
        Date.now() + 10 * 365 * 24 * 60 * 60 * 1000).toISOString()};path=/;`;
    }
};
exports.setColorSchemeCookie = setColorSchemeCookie;
/**
 * Returns a sane default for the initial color scheme. Tries to use the cookie on web before
 * falling back to the system default.
 */
const getInitialColorScheme = () => { var _a, _b; return (_b = (_a = (0, exports.getColorSchemeFromCookie)()) !== null && _a !== void 0 ? _a : react_native_1.Appearance.getColorScheme()) !== null && _b !== void 0 ? _b : "light"; };
exports.getInitialColorScheme = getInitialColorScheme;
