import React, { createContext, useContext, useEffect, useMemo } from "react";
import { Platform } from "react-native";
import { getInitialColorScheme, setColorSchemeCookie } from "./utils";
const DEFAULT_CONTEXT = {
    dark: getInitialColorScheme() === "dark",
    platform: Platform.OS,
};
const Context = createContext(DEFAULT_CONTEXT);
export const TailwindProvider = ({ dark, children }) => {
    // Memoize the context value to skip rerenders
    const value = useMemo(() => (Object.assign(Object.assign({}, DEFAULT_CONTEXT), { dark: Boolean(dark) })), [dark]);
    useEffect(() => {
        if (Platform.OS === "web" && typeof document !== "undefined") {
            document.body.classList.add(dark ? "rntwm-dark" : "rntwm-light");
            document.body.classList.remove(dark ? "rntwm-light" : "rntwm-dark");
            if (typeof dark !== "undefined") {
                // Persist the selection in a cookie
                setColorSchemeCookie(dark ? "dark" : "light");
            }
        }
    }, [dark]);
    return React.createElement(Context.Provider, { value: value }, children);
};
export const useTailwind = () => {
    const ctx = useContext(Context);
    return ctx;
};
