import { Appearance } from "react-native";
/**
 * Tries to retrieve the color scheme from a persisted cookie on web.
 */
export const getColorSchemeFromCookie = () => {
    var _a, _b;
    if (typeof document !== "undefined") {
        const cookie = (_b = (_a = document.cookie) === null || _a === void 0 ? void 0 : _a.match("(^|;)\\s*__rntwm_color_scheme\\s*=\\s*([^;]+)")) === null || _b === void 0 ? void 0 : _b.pop();
        if (cookie === "light" || cookie === "dark")
            return cookie;
    }
    return undefined;
};
/**
 * Tries to set the color scheme as a cookie on web.
 */
export const setColorSchemeCookie = (colorScheme) => {
    if (typeof document !== "undefined") {
        document.cookie = `__rntwm_color_scheme=${colorScheme};expires=${new Date(
        // Expire in 10 years, practically never
        Date.now() + 10 * 365 * 24 * 60 * 60 * 1000).toISOString()};path=/;`;
    }
};
/**
 * Returns a sane default for the initial color scheme. Tries to use the cookie on web before
 * falling back to the system default.
 */
export const getInitialColorScheme = () => { var _a, _b; return (_b = (_a = getColorSchemeFromCookie()) !== null && _a !== void 0 ? _a : Appearance.getColorScheme()) !== null && _b !== void 0 ? _b : "light"; };
